# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-03-02 15:28
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='RepeatableJob',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('name', models.CharField(max_length=128, unique=True, verbose_name='name')),
                ('callable', models.CharField(max_length=2048, verbose_name='callable')),
                ('enabled', models.BooleanField(default=True, verbose_name='enabled')),
                ('queue', models.CharField(choices=[(b'default', b'default'), (b'high', b'high'), (b'low', b'low')], max_length=16, verbose_name='queue')),
                ('job_id', models.CharField(blank=True, editable=False, max_length=128, null=True, verbose_name='job id')),
                ('scheduled_time', models.DateTimeField(verbose_name='scheduled time')),
                ('interval', models.PositiveIntegerField(verbose_name='interval')),
                ('interval_unit', models.CharField(choices=[('minutes', 'minutes'), ('hours', 'hours'), ('days', 'days'), ('weeks', 'weeks')], default='hours', max_length=12, verbose_name='interval unit')),
                ('repeat', models.PositiveIntegerField(blank=True, null=True, verbose_name='repeat')),
            ],
            options={
                'ordering': ('name',),
                'verbose_name': 'Repeatable Job',
                'verbose_name_plural': 'Repeatable Jobs',
            },
        ),
        migrations.CreateModel(
            name='ScheduledJob',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('name', models.CharField(max_length=128, unique=True, verbose_name='name')),
                ('callable', models.CharField(max_length=2048, verbose_name='callable')),
                ('enabled', models.BooleanField(default=True, verbose_name='enabled')),
                ('queue', models.CharField(choices=[(b'default', b'default'), (b'high', b'high'), (b'low', b'low')], max_length=16, verbose_name='queue')),
                ('job_id', models.CharField(blank=True, editable=False, max_length=128, null=True, verbose_name='job id')),
                ('scheduled_time', models.DateTimeField(verbose_name='scheduled time')),
            ],
            options={
                'ordering': ('name',),
                'verbose_name': 'Scheduled Job',
                'verbose_name_plural': 'Scheduled Jobs',
            },
        ),
    ]
