# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-14 12:45
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ('scheduler', '0003_remove_queue_choices'),
    ]

    operations = [
        migrations.CreateModel(
            name='CronJob',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('name', models.CharField(max_length=128, unique=True, verbose_name='name')),
                ('callable', models.CharField(max_length=2048, verbose_name='callable')),
                ('enabled', models.BooleanField(default=True, verbose_name='enabled')),
                ('queue', models.CharField(max_length=16, verbose_name='queue')),
                ('job_id', models.CharField(blank=True, editable=False, max_length=128, null=True, verbose_name='job id')),
                ('timeout', models.IntegerField(blank=True, help_text="Timeout specifies the maximum runtime, in seconds, for the job before it'll be considered 'lost'. Blank uses the default timeout.", null=True, verbose_name='timeout')),
                ('repeat', models.PositiveIntegerField(blank=True, null=True, verbose_name='repeat')),
                ('cron_string', models.CharField(help_text='Define the schedule in a crontab like syntax.', max_length=64, verbose_name='cron string')),
            ],
            options={
                'ordering': ('name',),
                'verbose_name': 'Cron Job',
                'verbose_name_plural': 'Cron Jobs',
            },
        ),
    ]
